==> jalview_2_07.tar.gz: jalview/src/jalview/appletgui/AnnotationPanel.java <==
/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;

public class AnnotationPanel
    extends Panel implements AdjustmentListener
{
  AlignViewport av;
  AlignmentPanel ap;
  int activeRow = -1;

  Vector activeRes;
  static String HELIX = "Helix";
  static String SHEET = "Sheet";
  static String LABEL = "Label";
  static String REMOVE = "Remove Annotation";
  static String COLOUR = "Colour";
  static Color HELIX_COLOUR = Color.red.darker();
  static Color SHEET_COLOUR = Color.green.darker().darker();

  Image image;
  Graphics gg;
  FontMetrics fm;
  int imgWidth = 0;

  boolean fastPaint = false;

  public static int GRAPH_HEIGHT = 40;

  public AnnotationPanel(AlignmentPanel ap)
  {
    this.ap = ap;
    av = ap.av;
    this.setLayout(null);
    adjustPanelHeight();

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseMoved(MouseEvent evt)
      {
        doMouseMoved(evt);
      }
    });

    // ap.annotationScroller.getVAdjustable().addAdjustmentListener( this );
  }


  public AnnotationPanel(AlignViewport av)
  {
    this.av = av;
  }


  public void adjustmentValueChanged(AdjustmentEvent evt)
  {
    ap.alabels.setScrollOffset( -evt.getValue());
  }

  public int adjustPanelHeight()
  {
    // setHeight of panels
    AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();


    int height = 0;
    if (aa != null)
    {
      for (int i = 0; i < aa.length; i++)
      {
        if (!aa[i].visible)
        {
          continue;
        }

        aa[i].height = 0;

        if (aa[i].hasText)
        {
          aa[i].height += av.charHeight;
        }
        if (aa[i].hasIcons)
        {
          aa[i].height += 16;
        }

        if (aa[i].isGraph)
        {
          aa[i].height += GRAPH_HEIGHT;
        }

        if (aa[i].height == 0)
        {
          aa[i].height = 20;
        }
        height += aa[i].height;
      }
    }
    else
    {
      height = 20;
    }

    this.setSize(getSize().width, height);
    if(ap!=null)
       ap.annotationScroller.setSize(getSize().width, height);


    repaint();

    return height;

  }

  public void addEditableColumn(int i)
  {
    if (activeRow == -1)
    {
      AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
      if(aa ==null)
        return;

      for (int j = 0; j < aa.length; j++)
      {
        if (aa[j].editable)
        {
          activeRow = j;
          break;
        }
      }
    }

    if (activeRes == null)
    {
      activeRes = new Vector();
      activeRes.addElement(String.valueOf(i));
      return;
    }

    activeRes.addElement(String.valueOf(i));
  }

  public void doMouseMoved(MouseEvent evt)
  {
    AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
    if (aa == null)
    {
      return;
    }

    int row = -1;
    int height = 0;
    for (int i = 0; i < aa.length; i++)
    {

      if (aa[i].visible)
      {
        height += aa[i].height;
      }

      if (evt.getY() < height)
      {
        row = i;
        break;
      }
    }

    int res = evt.getX() / av.getCharWidth() + av.getStartRes();
    if (row > -1 && res < aa[row].annotations.length && aa[row].annotations[res] != null)
    {
      StringBuffer text = new StringBuffer("Sequence position " + (res + 1) +
                                           "  " +
                                           aa[row].annotations[res].description);
      ap.alignFrame.statusBar.setText(text.toString());
    }
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    g.setColor(Color.white);
    g.fillRect(0,0, getSize().width, getSize().height);
    imgWidth = (av.endRes - av.startRes + 1) * av.charWidth;

    if (image == null || imgWidth != image.getWidth(this))
    {
      image = createImage(imgWidth, ap.annotationPanel.getSize().height);
      gg = image.getGraphics();
      gg.setFont(av.getFont());
      fm = gg.getFontMetrics();
      fastPaint = false;
    }

    if (fastPaint)
    {
      g.drawImage(image, 0, 0, this);
      fastPaint = false;
      return;
    }

    drawComponent(gg, av.startRes, av.endRes + 1);
    g.drawImage(image, 0, 0, this);
  }

  public void fastPaint(int horizontal)
  {
    if (horizontal == 0
        || av.alignment.getAlignmentAnnotation() == null
        || av.alignment.getAlignmentAnnotation().length < 1
        )
    {
      repaint();
      return;
    }

    gg.copyArea(0, 0, imgWidth, getSize().height, -horizontal * av.charWidth, 0);
    int sr = av.startRes, er = av.endRes + 1, transX = 0;

    if (horizontal > 0) // scrollbar pulled right, image to the left
    {
      transX = (er - sr - horizontal) * av.charWidth;
      sr = er - horizontal;
    }
    else if (horizontal < 0)
    {
      er = sr - horizontal;
    }

    gg.translate(transX, 0);

    drawComponent(gg, sr, er);

    gg.translate( -transX, 0);

    fastPaint = true;
    repaint();
  }

  public void drawComponent(Graphics g, int startRes, int endRes)
  {
    g.setFont(av.getFont());

    if (fm == null)
      fm = g.getFontMetrics();


      g.setColor(Color.white);
      g.fillRect(0, 0, (endRes - startRes) * av.charWidth, getSize().height);

      if ((av.alignment.getAlignmentAnnotation() == null) ||
              (av.alignment.getAlignmentAnnotation().length < 1))
      {
          g.setColor(Color.white);
          g.fillRect(0, 0, getSize().width, getSize().height);
          g.setColor(Color.black);
          g.drawString("Alignment has no annotations", 20, 15);

          return;
      }

      AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();

      int j;
      int x = 0;
      int y = 0;
      char[] lastSS = new char[aa.length];
      int[] lastSSX = new int[aa.length];
      int iconOffset = av.charHeight / 2;
      boolean validRes = false;

      //\u03B2 \u03B1
      for (int i = 0; i < aa.length; i++)
      {
          AlignmentAnnotation row = aa[i];

          if (!row.visible)
          {
              continue;
          }


          if (row.isGraph)
          {
              // this is so that we draw the characters below the graph
              y += row.height;

              if (row.hasText)
              {
                  y -= av.charHeight;
              }
          }

          if (row.hasText)
          {
              iconOffset = av.charHeight / 2;
          }
          else
          {
              iconOffset = 0;
          }

          for (j = startRes; j < endRes; j++)
          {
              if ((row.annotations.length <= j) ||
                      (row.annotations[j] == null))
              {
                  validRes = false;
              }
              else
              {
                  validRes = true;
              }

              x = (j - startRes) * av.charWidth;


              if (validRes &&
                      (row.annotations[j].displayCharacter.length() > 0))
              {

                  int charOffset = (av.charWidth -
                      fm.charWidth(row.annotations[j].displayCharacter.charAt(
                              0))) / 2;
                  g.setColor(row.annotations[j].colour);

                  if (j == 0 || row.isGraph)
                  {
                      g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                          y + iconOffset + 3);
                  }
                  else if (((row.annotations[j - 1] == null) ||
                          (row.annotations[j].displayCharacter != row.annotations[j -
                          1].displayCharacter)))
                  {
                      g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                          y + iconOffset + 3);
                  }
              }

              if (row.hasIcons)
              {
                  if (!validRes ||
                          (row.annotations[j].secondaryStructure != lastSS[i]))
                  {
                      switch (lastSS[i])
                      {
                      case 'H':
                          g.setColor(HELIX_COLOUR);
                          g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                              x - lastSSX[i], 7, 8, 8);

                          break;

                      case 'E':
                          g.setColor(SHEET_COLOUR);
                          g.fillRect(lastSSX[i], y + 4 + iconOffset,
                              x - lastSSX[i] - 4, 7);
                          g.fillPolygon(new int[] { x - 4, x - 4, x },
                              new int[]
                              {
                                  y + iconOffset, y + 14 + iconOffset,
                                  y + 8 + iconOffset
                              }, 3);

                          break;

                      case 'C':
                          break;

                      default:
                          g.setColor(Color.gray);
                          g.fillRect(lastSSX[i], y + 6 + iconOffset,
                              x - lastSSX[i], 2);

                          break;
                      }

                      if (validRes)
                      {
                          lastSS[i] = row.annotations[j].secondaryStructure;
                      }
                      else
                      {
                          lastSS[i] = ' ';
                      }

                      lastSSX[i] = x;
                  }
              }

              if (validRes && row.isGraph)
              {
                  g.setColor(new Color(0, 0, 180));

                  int height = (int) ((row.annotations[j].value / row.graphMax) * GRAPH_HEIGHT);

                  if (row.windowLength > 1)
                  {
                      int total = 0;

                      for (int i2 = j - (row.windowLength / 2);
                              i2 < (j + (row.windowLength / 2)); i2++)
                      {
                          if ((i2 < 0) || (i2 >= av.alignment.getWidth()))
                          {
                              continue;
                          }

                          total += row.annotations[i2].value;
                      }

                      total /= row.windowLength;
                      height = (int) ((total / row.graphMax) * GRAPH_HEIGHT);
                  }

                  g.setColor(row.annotations[j].colour);
                  g.fillRect(x, y - height, av.charWidth, height);
              }
          }

          x += av.charWidth;

          if (row.hasIcons)
          {
              switch (lastSS[i])
              {
              case 'H':
                  g.setColor(HELIX_COLOUR);
                  g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                      x - lastSSX[i], 7, 8, 8);

                  break;

              case 'E':
                  g.setColor(SHEET_COLOUR);
                  g.fillRect(lastSSX[i], y + 4 + iconOffset,
                      x - lastSSX[i] - 4, 7);
                  g.fillPolygon(new int[] { x - 4, x - 4, x },
                      new int[]
                      {
                          y + iconOffset, y + 14 + iconOffset,
                          y + 7 + iconOffset
                      }, 3);

                  break;

              case 'C':
                  break;

              default:
                  g.setColor(Color.gray);
                  g.fillRect(lastSSX[i], y + 6 + iconOffset, x - lastSSX[i], 2);

                  break;
              }
          }

          if (row.isGraph && row.hasText)
          {
              y += av.charHeight;
          }

          if (!row.isGraph)
          {
              y += aa[i].height;
          }
        }
  }

  // used by overview window
  public void drawGraph(Graphics g, AlignmentAnnotation aa, int width, int y, int sRes, int eRes)
  {
      g.setColor(Color.white);
      g.fillRect(0, 0, width, y);
      g.setColor(new Color(0, 0, 180));

      int x = 0, height;

      for (int j = sRes; j < eRes; j++)
      {
          g.setColor(new Color(0, 0, 180));

          height = (int) ((aa.annotations[j].value / aa.graphMax) * GRAPH_HEIGHT);
          if(height>y)
              height = y;
          g.fillRect(x, y - height, av.charWidth, height);
          x += av.charWidth;
      }
    }
}

==> jalview_2_08b.tar.gz: jalview/src/jalview/appletgui/AnnotationPanel.java <==
/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;

public class AnnotationPanel
    extends Panel implements AdjustmentListener
{
  AlignViewport av;
  AlignmentPanel ap;
  int activeRow = -1;

  Vector activeRes;
  static String HELIX = "Helix";
  static String SHEET = "Sheet";
  static String LABEL = "Label";
  static String REMOVE = "Remove Annotation";
  static String COLOUR = "Colour";
  static Color HELIX_COLOUR = Color.red.darker();
  static Color SHEET_COLOUR = Color.green.darker().darker();

  Image image;
  Graphics gg;
  FontMetrics fm;
  int imgWidth = 0;

  boolean fastPaint = false;

  public static int GRAPH_HEIGHT = 40;

  public AnnotationPanel(AlignmentPanel ap)
  {
    this.ap = ap;
    av = ap.av;
    this.setLayout(null);
    adjustPanelHeight();

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseMoved(MouseEvent evt)
      {
        doMouseMoved(evt);
      }
    });

    // ap.annotationScroller.getVAdjustable().addAdjustmentListener( this );
  }


  public AnnotationPanel(AlignViewport av)
  {
    this.av = av;
  }


  public void adjustmentValueChanged(AdjustmentEvent evt)
  {
    ap.alabels.setScrollOffset( -evt.getValue());
  }

  public int adjustPanelHeight()
  {
    // setHeight of panels
    AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();


    int height = 0;
    if (aa != null)
    {
      for (int i = 0; i < aa.length; i++)
      {
        if (!aa[i].visible)
        {
          continue;
        }

        aa[i].height = 0;

        if (aa[i].hasText)
        {
          aa[i].height += av.charHeight;
        }
        if (aa[i].hasIcons)
        {
          aa[i].height += 16;
        }

        if (aa[i].graph>0)
        {
          aa[i].height += GRAPH_HEIGHT;
        }

        if (aa[i].height == 0)
        {
          aa[i].height = 20;
        }
        height += aa[i].height;
      }
    }
    else
    {
      height = 20;
    }

    this.setSize(getSize().width, height);
    if(ap!=null)
       ap.annotationScroller.setSize(getSize().width, height);


    repaint();

    return height;

  }

  public void addEditableColumn(int i)
  {
    if (activeRow == -1)
    {
      AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
      if(aa ==null)
        return;

      for (int j = 0; j < aa.length; j++)
      {
        if (aa[j].editable)
        {
          activeRow = j;
          break;
        }
      }
    }

    if (activeRes == null)
    {
      activeRes = new Vector();
      activeRes.addElement(String.valueOf(i));
      return;
    }

    activeRes.addElement(String.valueOf(i));
  }

  public void doMouseMoved(MouseEvent evt)
  {
    AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();
    if (aa == null)
    {
      return;
    }

    int row = -1;
    int height = 0;
    for (int i = 0; i < aa.length; i++)
    {

      if (aa[i].visible)
      {
        height += aa[i].height;
      }

      if (evt.getY() < height)
      {
        row = i;
        break;
      }
    }

    int res = evt.getX() / av.getCharWidth() + av.getStartRes();
    if (row > -1 && res < aa[row].annotations.length && aa[row].annotations[res] != null)
    {
      StringBuffer text = new StringBuffer("Sequence position " + (res + 1) +
                                           "  " +
                                           aa[row].annotations[res].description);
      ap.alignFrame.statusBar.setText(text.toString());
    }
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    imgWidth = (av.endRes - av.startRes + 1) * av.charWidth;

    if (image == null || imgWidth != image.getWidth(this))
    {
      image = createImage(imgWidth, ap.annotationPanel.getSize().height);
      gg = image.getGraphics();
      gg.setFont(av.getFont());
      fm = gg.getFontMetrics();
      fastPaint = false;
    }

    if (fastPaint)
    {
      g.drawImage(image, 0, 0, this);
      fastPaint = false;
      return;
    }

    drawComponent(gg, av.startRes, av.endRes + 1);
    g.drawImage(image, 0, 0, this);
  }

  public void fastPaint(int horizontal)
  {
    if (horizontal == 0
        || av.alignment.getAlignmentAnnotation() == null
        || av.alignment.getAlignmentAnnotation().length < 1
        )
    {
      repaint();
      return;
    }

    gg.copyArea(0, 0, imgWidth, getSize().height, -horizontal * av.charWidth, 0);
    int sr = av.startRes, er = av.endRes + 1, transX = 0;

    if (horizontal > 0) // scrollbar pulled right, image to the left
    {
      transX = (er - sr - horizontal) * av.charWidth;
      sr = er - horizontal;
    }
    else if (horizontal < 0)
    {
      er = sr - horizontal;
    }

    gg.translate(transX, 0);

    drawComponent(gg, sr, er);

    gg.translate( -transX, 0);

    fastPaint = true;
    repaint();
  }

  /**
   * DOCUMENT ME!
   *
   * @param g DOCUMENT ME!
   * @param startRes DOCUMENT ME!
   * @param endRes DOCUMENT ME!
   */
  public void drawComponent(Graphics g, int startRes, int endRes)
  {
    g.setFont(av.getFont());

    if (fm == null)
      fm = g.getFontMetrics();


      g.setColor(Color.white);
      g.fillRect(0, 0, (endRes - startRes) * av.charWidth, getSize().height);

      if ((av.alignment.getAlignmentAnnotation() == null) ||
              (av.alignment.getAlignmentAnnotation().length < 1))
      {
          g.setColor(Color.white);
          g.fillRect(0, 0, getSize().width, getSize().height);
          g.setColor(Color.black);
          g.drawString("Alignment has no annotations", 20, 15);

          return;
      }

      AlignmentAnnotation[] aa = av.alignment.getAlignmentAnnotation();

      int j;
      int x = 0;
      int y = 0;
      char[] lastSS = new char[aa.length];
      int[] lastSSX = new int[aa.length];
      int iconOffset = av.charHeight / 2;
      boolean validRes = false;

      boolean [] graphGroupDrawn = new boolean[aa.length];


      //\u03B2 \u03B1
      for (int i = 0; i < aa.length; i++)
      {
          AlignmentAnnotation row = aa[i];

          if (!row.visible)
          {
              continue;
          }



          if (row.graph>0)
          {
              if(row.graphGroup>-1 && graphGroupDrawn[ row.graphGroup ] )
                continue;

              // this is so that we draw the characters below the graph
              y += row.height;

              if (row.hasText)
              {
                  y -= av.charHeight;
              }
          }

          if (row.hasText)
          {
              iconOffset = av.charHeight / 2;
          }
          else
          {
              iconOffset = 0;
          }

          for (j = startRes; j < endRes; j++)
          {
              if ((row.annotations.length <= j) ||
                      (row.annotations[j] == null))
              {
                  validRes = false;
              }
              else
              {
                  validRes = true;
              }

              x = (j - startRes) * av.charWidth;

              if (activeRow == i)
              {
                  g.setColor(Color.red);

                  if (activeRes != null)
                  {
                      for (int n = 0; n < activeRes.size(); n++)
                      {
                          int v = Integer.parseInt(activeRes.elementAt(n).toString());

                          if (v == j)
                          {
                              g.fillRect((j - startRes) * av.charWidth, y,
                                  av.charWidth, row.height);
                          }
                      }
                  }
              }

              if (validRes &&
                      (row.annotations[j].displayCharacter.length() > 0))
              {

                  int charOffset = (av.charWidth -
                      fm.charWidth(row.annotations[j].displayCharacter.charAt(
                              0))) / 2;
                  g.setColor(row.annotations[j].colour);

                  if (j == 0 || row.graph>0)
                  {
                      g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                          y + iconOffset + 3);
                  }
                  else if (((row.annotations[j - 1] == null) ||
                          (!row.annotations[j].displayCharacter.equals(
                           row.annotations[j - 1].displayCharacter))))
                  {
                      g.drawString(row.annotations[j].displayCharacter, x+charOffset,
                          y + iconOffset + 3);
                  }
              }

              if (row.hasIcons)
              {
                  if (!validRes ||
                          (row.annotations[j].secondaryStructure != lastSS[i]))
                  {
                      switch (lastSS[i])
                      {
                      case 'H':
                          g.setColor(HELIX_COLOUR);
                          g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                              x - lastSSX[i], 7, 8, 8);

                          break;

                      case 'E':
                          g.setColor(SHEET_COLOUR);
                          g.fillRect(lastSSX[i], y + 4 + iconOffset,
                              x - lastSSX[i] - 4, 7);
                          g.fillPolygon(new int[] { x - 4, x - 4, x },
                              new int[]
                              {
                                  y + iconOffset, y + 14 + iconOffset,
                                  y + 8 + iconOffset
                              }, 3);

                          break;


                      default:
                          g.setColor(Color.gray);
                          g.fillRect(lastSSX[i], y + 6 + iconOffset,
                              x - lastSSX[i], 2);

                          break;
                      }

                      if (validRes)
                      {
                          lastSS[i] = row.annotations[j].secondaryStructure;
                      }
                      else
                      {
                          lastSS[i] = ' ';
                      }

                      lastSSX[i] = x;
                  }
              }
          }

          x += av.charWidth;

          if (row.hasIcons)
          {
              switch (lastSS[i])
              {
              case 'H':
                  g.setColor(HELIX_COLOUR);
                  g.fillRoundRect(lastSSX[i], y + 4 + iconOffset,
                      x - lastSSX[i], 7, 8, 8);

                  break;

              case 'E':
                  g.setColor(SHEET_COLOUR);

                  if (row.annotations.length > endRes
                      && row.annotations[endRes].secondaryStructure != 'E')
                  {
                    g.fillRect(lastSSX[i], y + 4 + iconOffset,
                               x - lastSSX[i] - 4, 7);
                    g.fillPolygon(new int[]
                                  {x - 4, x - 4, x},
                                  new int[]
                                  {
                                  y + iconOffset, y + 14 + iconOffset,
                                  y + 7 + iconOffset
                    }, 3);
                  }
                  else
                    g.fillRect(lastSSX[i], y + 4 + iconOffset,
                               x - lastSSX[i], 7);

                  break;

              case 'C':
                  break;

              default:
                  g.setColor(Color.gray);
                  g.fillRect(lastSSX[i], y + 6 + iconOffset, x - lastSSX[i], 2);

                  break;
              }
          }

          if (row.graph>0)
          {
              if(row.graph == AlignmentAnnotation.LINE_GRAPH )
              {
                if(row.graphGroup>-1 && !graphGroupDrawn[row.graphGroup])
                 {
                   float groupmax=-999999, groupmin=9999999;
                   for(int gg=0; gg<aa.length; gg++)
                   {
                     if(aa[gg].graphGroup!=row.graphGroup)
                       continue;

                     if(aa[gg]!=row)
                       aa[gg].visible = false;

                     if(aa[gg].graphMax>groupmax)
                       groupmax = aa[gg].graphMax;
                     if(aa[gg].graphMin<groupmin)
                       groupmin = aa[gg].graphMin;
                   }

                   for (int gg = 0; gg < aa.length; gg++)
                   {
                     if (aa[gg].graphGroup == row.graphGroup)
                     {
                       drawLineGraph(g, aa[gg], startRes, endRes, y,
                                     groupmin, groupmax,
                                     row.graphHeight);
                     }
                   }

                   graphGroupDrawn[ row.graphGroup ] = true;
                 }
                 else
                   drawLineGraph(g, row, startRes, endRes,
                                 y, row.graphMin, row.graphMax, row.graphHeight  );
              }
              else if(row.graph == AlignmentAnnotation.BAR_GRAPH )
                 drawBarGraph(g, row, startRes, endRes,
                              row.graphMin, row.graphMax, y);
          }

          if (row.graph>0 && row.hasText)
          {
              y += av.charHeight;
          }

          if (row.graph==0)
          {
              y += aa[i].height;
          }
      }
  }

  public void drawLineGraph(Graphics g, AlignmentAnnotation aa,
                            int sRes, int eRes,
                            int y,
                            float min, float max,
                            int graphHeight)
  {
    if(sRes>aa.annotations.length)
      return;


    eRes = Math.min(eRes, aa.annotations.length);

    int x = 0;

    //Adjustment for fastpaint to left
    if(eRes<av.endRes)
      eRes++;

    if(sRes==0)
    {
      sRes++;
      x+=av.charWidth;
    }

    int y1=y, y2=y;
    float range = max - min;

    ////Draw origin
    if(min<0)
      y2 = (int)(y - (0-min / range)*graphHeight);

    g.setColor(Color.gray);
    g.drawLine(x-av.charWidth,y2,(eRes-sRes)*av.charWidth,y2);


    for (int j = sRes; j < eRes; j++)
    {
      if(aa.annotations[j]==null || aa.annotations[j-1]==null)
      {
        x+=av.charWidth;
        continue;
      }
        g.setColor(aa.annotations[j].colour);
        y1 = y - (int) (((aa.annotations[j-1].value-min) / range) * graphHeight);
        y2 = y - (int) (((aa.annotations[j].value-min) / range) * graphHeight);
        g.drawLine(x-av.charWidth/2, y1, x+av.charWidth/2, y2);
        x += av.charWidth;
     }


     if(aa.threshold!=null)
     {
         g.setColor(aa.threshold.colour);
         y2 = (int)(y - ((aa.threshold.value-min) / range)*aa.graphHeight);
         g.drawLine(0,y2,(eRes-sRes)*av.charWidth,y2);
     }

  }

  public void drawBarGraph(Graphics g, AlignmentAnnotation aa,
                           int sRes, int eRes,
                           float min, float max,
                           int y)
  {
    if(sRes>aa.annotations.length)
      return;


    eRes = Math.min(eRes, aa.annotations.length);

    int x=0, y1, y2;

    float range = max - min;

    y1 = y2 = y;

    if(min<0)
      y2 = (int)(y - (0-min / (range))*aa.graphHeight);

    g.setColor(Color.gray);

    g.drawLine(x,y2,(eRes-sRes)*av.charWidth,y2);

    for (int j = sRes; j < eRes; j++)
    {

      if (aa.annotations[j] == null)
      {
        x += av.charWidth;
        continue;
      }

        g.setColor(aa.annotations[j].colour);
        y1 = y - (int) (((aa.annotations[j].value-min) / (range)) * aa.graphHeight);

        if(y1-y2>0)
          g.fillRect(x, y2, av.charWidth, y1-y2 );
        else
          g.fillRect(x, y1, av.charWidth, y2-y1 );

        x += av.charWidth;
    }


    if(aa.threshold!=null)
    {
        g.setColor(aa.threshold.colour);
        y2 = (int)(y - ((aa.threshold.value-min) / range)*aa.graphHeight);
        g.drawLine(0,y2,(eRes-sRes)*av.charWidth,y2);
    }


  }

  // used by overview window
  public void drawGraph(Graphics g, AlignmentAnnotation aa, int width, int y, int sRes, int eRes)
  {
      g.setColor(Color.white);
      g.fillRect(0, 0, width, y);
      g.setColor(new Color(0, 0, 180));

      int x = 0, height;

      for (int j = sRes; j < eRes; j++)
      {
          g.setColor(aa.annotations[j].colour);

          height = (int) ((aa.annotations[j].value / aa.graphMax) * GRAPH_HEIGHT);
          if(height>y)
              height = y;
          g.fillRect(x, y - height, av.charWidth, height);
          x += av.charWidth;
      }
    }
}
